"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _l7Utils = require("@antv/l7-utils");
class TextureService {
  constructor(layer) {
    (0, _defineProperty2.default)(this, "layer", void 0);
    (0, _defineProperty2.default)(this, "rendererService", void 0);
    (0, _defineProperty2.default)(this, "colorTexture", void 0);
    (0, _defineProperty2.default)(this, "key", void 0);
    this.layer = layer;
    const container = this.layer.getContainer();
    this.rendererService = container.rendererService;
  }
  getColorTexture(colorRamp, domain) {
    // TODO 支持传入图片
    const currentkey = this.getTextureKey(colorRamp, domain);
    if (this.key === currentkey) {
      return this.colorTexture;
    } else {
      this.createColorTexture(colorRamp, domain);
    }
    this.key = currentkey;
    return this.colorTexture;
  }
  createColorTexture(colorRamp, domain) {
    const {
      createTexture2D
    } = this.rendererService;
    const imageData = this.getColorRampBar(colorRamp, domain);
    const texture = createTexture2D({
      data: new Uint8Array(imageData.data),
      width: imageData.width,
      height: imageData.height,
      flipY: false,
      unorm: true
    });
    this.colorTexture = texture;
    return texture;
  }
  setColorTexture(texture, colorRamp, domain) {
    this.key = this.getTextureKey(colorRamp, domain);
    this.colorTexture = texture;
  }
  destroy() {
    var _this$colorTexture;
    (_this$colorTexture = this.colorTexture) === null || _this$colorTexture === void 0 || _this$colorTexture.destroy();
  }
  getColorRampBar(colorRamp, domain) {
    switch (colorRamp.type) {
      case 'cat':
        return (0, _l7Utils.generateCatRamp)(colorRamp);
      case 'quantize':
        return (0, _l7Utils.generateQuantizeRamp)(colorRamp);
      case 'custom':
        return (0, _l7Utils.generateCustomRamp)(colorRamp, domain);
      case 'linear':
        return (0, _l7Utils.generateLinearRamp)(colorRamp, domain);
      default:
        return (0, _l7Utils.generateColorRamp)(colorRamp);
    }
  }
  getTextureKey(colorRamp, domain) {
    var _colorRamp$positions;
    return `${colorRamp.colors.join('_')}_${colorRamp === null || colorRamp === void 0 || (_colorRamp$positions = colorRamp.positions) === null || _colorRamp$positions === void 0 ? void 0 : _colorRamp$positions.join('_')}_${colorRamp.type}_${domain === null || domain === void 0 ? void 0 : domain.join('_')}`;
  }
}
exports.default = TextureService;